﻿using System.Reflection;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples;

namespace DssApiSamples.ConsoleRunner
{
    class Program
    {
        static void Main()
        {
            var writer = new ConsoleMessageWriter();

            writer.WriteLine(CoreUtility.GetHostTitle(Assembly.GetExecutingAssembly()));
            writer.WriteLine("API URL: " + ApiUtility.ApiUrl);
            writer.WriteLine();

            //run all samples
            var samples = CoreUtility.GetSamples();
            foreach (var sample in samples)
                try
                {
                    sample.Run(writer);
                }
                catch (ClientErrorException ex)
                {
                    //display details
                    writer.WriteLine();
                    writer.WriteLine(ex.Message);
                    writer.WriteLine("CLIENT ERROR:");
                    writer.WriteLine("  " + ex.Message);
                    writer.WriteLine("  Messages =");
                    foreach (var message in ex.Error.Messages)
                        writer.WriteLine("  " + message);

                    //rethrow to end app, unless the error is for making a call that is OBSOLETE
                    if (ex.Error.ErrorCode != "CE-OBS")
                        throw;
                }
                catch (ServerErrorException ex)
                {
                    //display details
                    writer.WriteLine();
                    writer.WriteLine("SERVER ERROR:");
                    writer.WriteLine("  " + ex.Message);
                    writer.WriteLine("  ErrorInstanceId = {0}", ex.Error.ErrorInstanceId);
                    writer.WriteLine("  Host = {0}", ex.Error.Host);

                    //rethrow to end app
                    throw;
                }
                catch (InvalidOutputFieldsException ex)
                {
                    //display details
                    writer.WriteLine();
                    writer.WriteLine("INVALID OUTPUT FIELDS:");
                    writer.WriteLine("  " + ex.Message);
                    foreach (var field in ex.Fields)
                        writer.WriteLine("  " + field);

                    //rethrow to end app
                    throw;
                }
                catch (FaultException ex)
                {
                    //display details
                    writer.WriteLine();
                    writer.WriteLine("GENERAL SOAP FAULT:");
                    writer.WriteLine("  " + ex.Message);
                    writer.WriteLine("  Fault Code = {0}", ex.Code.Name);

                    //rethrow to end app
                    throw;
                }
        }
    }
}
