﻿using System.IO;
using DssApiSamples.Core;

namespace DssApiSamples.GuiRunner
{
    /// <summary>
    /// An <see cref="IMessageWriter"/> class that writes messages to a file as well as a 
    /// wrapped <see cref="IMessageWriter"/> instance provided by the host application.
    /// </summary>
    public sealed class FileMessageWriter : IMessageWriter
    {
        private readonly IMessageWriter _innerWriter;
        private readonly StreamWriter _file;

        public FileMessageWriter(StreamWriter file, IMessageWriter innerWriter)
        {
            _file = file;
            _innerWriter = innerWriter;
        }

        public void WriteLine(string message)
        {
            _file.WriteLine(message);
            _innerWriter.WriteLine(message);
        }

        public void WriteLine(string format, params object[] args)
        {
            _file.WriteLine(format, args);
            _innerWriter.WriteLine(format, args);
        }

        public void WriteLine()
        {
            _file.WriteLine();
            _innerWriter.WriteLine();
        }
    }
}