﻿using System;
using System.ComponentModel;
using DssApiSamples.Core;
using DssApiSamples.GuiRunner.Async;

namespace DssApiSamples.GuiRunner
{
    /// <summary>
    /// An <see cref="IMessageWriter"/> class that sends messages to a <see cref="BackgroundWorker"/> component on a WinForm.
    /// </summary>
    public class GuiMessageWriter : IMessageWriter
    {
        private readonly BackgroundWorker _backgroundWorker;

        public GuiMessageWriter(BackgroundWorker backgroundWorker)
        {
            _backgroundWorker = backgroundWorker;
        }

        public void WriteLine(string message)
        {
            var text = message + Environment.NewLine;
            WriteProgress(text);
        }

        public void WriteLine(string format, params object[] args)
        {
            var message = string.Format(format, args);
            WriteLine(message);
        }

        public void WriteLine()
        {
            WriteLine(string.Empty);
        }

        private void WriteProgress(string text)
        {
            var progress = new OutputMessageNotification {Text = text};
            _backgroundWorker.ReportProgress(0, progress);
        }
    }
}
