﻿using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Advanced
{
    public class ExecuteSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    writer.WriteLine("Performing a call to the API using the ApiUtility.Execute method");

                    // Create and initialize the request object. 
                    var request = new GetInstrumentIdentifierTypesRequest
                                      {
                                          CredentialsHeader = credentials
                                      };

                    //Wrap the call to the API using ApiUtility.Execute, 
                    //which removes the need for the try/catch and also calls ApiUtility.Retry
                    var response = ApiUtility.Execute(
                        () => proxy.GetInstrumentIdentifierTypes(request));

                    // Display the response.
                    foreach (var type in response.GetInstrumentIdentifierTypesResult)
                        writer.WriteLine("  {0}: {1}",
                                     type.Value,
                                     type.Description);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}