using System;
using System.Runtime.Serialization;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples
{
    /// <summary>
    /// The <see cref="Exception"/> that is thrown by the <see cref="ApiUtility.ConvertFaultException"/> method when 
    /// the SOAP fault represents a client error.
    /// </summary>
    [Serializable]
    public class ClientErrorException : Exception
    {
        #region Standard exception constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ClientErrorException"/> class.
        /// </summary>
        public ClientErrorException() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ClientErrorException"/> class with a specified error message.
        /// </summary>
        public ClientErrorException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ClientErrorException"/> class with a specified error message 
        /// and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        public ClientErrorException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ClientErrorException"/> class with serialized data.
        /// </summary>
        public ClientErrorException(SerializationInfo info, StreamingContext context) : base(info, context) { }

        #endregion
        
        /// <summary>
        /// Initalizes a new instance of a <see cref="ClientErrorException"/> 
        /// with the specified message and <see cref="ClientError"/>.
        /// </summary>
        public ClientErrorException(string message, ClientError error) : base(message)
        {
            Error = error;
        }

        /// <summary>
        /// Gets the <see cref="ClientError"/> that caused the exception.
        /// </summary>
        public ClientError Error { get; private set; }
    }
}