﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.DataDictionary
{
    public class CounterpartySample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string counterpartyAudit = "CPA";

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Getting all Counterparty fields that:");
                    writer.WriteLine("- are associated with the Counterparty Audit ({0}) extraction type", counterpartyAudit);

                    #endregion

                    #region Make data dictionary request

                    // Create and initialize the request object. 
                    var request =
                        new DefineRequest
                            {
                                CredentialsHeader = credentials,
                                request = new DataDictionaryRequestCounterparty
                                              {
                                                  // NOTE: call GetCounterpartyExtractionTypes operation to see all extraction types.
                                                  CounterpartyExtractionType = counterpartyAudit
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    DefineResponse response;
                    try
                    {
                        response = proxy.Define(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} fields found:", response.DefineResult.Fields.Length);
                    foreach (var field in response.DefineResult.Fields)
                    {
                        writer.WriteLine("  {0}: {1}", field.Name, field.Description);
                        writer.WriteLine("  - Asset Types = {0}", string.Join(", ", field.AssetTypes));
                        writer.WriteLine("  - Corax Events = {0}", string.Join(", ", field.CoraxEvents));
                        writer.WriteLine("  - Bond Schedule Types = {0}", string.Join(", ", field.BondScheduleTypes));
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}