﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Discoverability
{
    public class GetCmoAbsSubGroupExtTypesSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            //build reusable API proxy, disposing it when done
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    //create reusable credentials object
                    var credentials = ApiUtility.CreateCredentials();

                    #region Display sample description

                    writer.WriteLine("Invoking discoverability method...");

                    #endregion

                    #region Perform discoverability method call

                    // Create and initialize the request object.
                    var request =
                        new GetCmoAbsSubGroupExtTypesRequest
                            {
                                CredentialsHeader = credentials
                            };

                    //call API to get response, handling API errors
                    GetCmoAbsSubGroupExtTypesResponse response;
                    try
                    {
                        response = proxy.GetCmoAbsSubGroupExtTypes(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    foreach (var valueInfo in response.GetCmoAbsSubGroupExtTypesResult)
                        writer.WriteLine("  {0}: {1}",
                                     valueInfo.Value,
                                     valueInfo.Description);

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}