﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Discoverability
{
    public class GetCoraxEventsSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Display sample description

                    writer.WriteLine("Invoking discoverability method...");

                    #endregion

                    #region Perform discoverability method call

                    // Create and initialize the request object.
                    var request =
                        new GetCoraxEventsRequest
                            {
                                CredentialsHeader = credentials
                            };

                    // Call the API to get the response, handling API errors.
                    GetCoraxEventsResponse response;
                    try
                    {
                        response = proxy.GetCoraxEvents(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    foreach (var eventInfo in response.GetCoraxEventsResult)
                    {
                        writer.WriteLine("  {0}: {1}, Events: {2}",
                                     eventInfo.Value,
                                     eventInfo.Description,
                                     string.Join(", ", eventInfo.Constraints));
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}