﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Discoverability
{
    public class GetInstrumentExtractionTypesSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Display sample description

                    writer.WriteLine("Invoking discoverability method...");

                    #endregion

                    #region Perform discoverability method call

                    // Create and initialize the request object. 
                    var request =
                        new GetInstrumentExtractionTypesRequest
                            {
                                CredentialsHeader = credentials
                            };

                    // Call the API to get the response, handling API errors.
                    GetInstrumentExtractionTypesResponse response;
                    try
                    {
                        response = proxy.GetInstrumentExtractionTypes(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    foreach (var extractionType in response.GetInstrumentExtractionTypesResult)
                    {
                        writer.WriteLine("{0} = {1}", extractionType.Value, extractionType.Description);
                        writer.WriteLine("- Asset Types =");
                        foreach (var assetType in extractionType.AssetTypes)
                            writer.WriteLine("  {0}: {1}",
                                         assetType.Value,
                                         assetType.Description);
                        writer.WriteLine("- MaximumInstrumentsAllowed = {0}", extractionType.MaximumInstrumentsAllowed);
                        writer.WriteLine("- TimePeriodForLimits = {0}", extractionType.TimePeriodForLimits);
                        writer.WriteLine("- MaximumExtractionsAllowedPerTimePeriod = {0}",
                                     extractionType.MaximumExtractionsAllowedPerTimePeriod);
                        writer.WriteLine("- MaximumInstrumentsAllowedPerTimePeriod = {0}",
                                     extractionType.MaximumInstrumentsAllowedPerTimePeriod);
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}