﻿using System.Linq;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Discoverability
{
    public class GetInstrumentSearchTypesSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Display sample description

                    writer.WriteLine("Invoking discoverability method...");

                    #endregion

                    #region Perform discoverability method call

                    // Create and initialize the request object. 
                    var request =
                        new GetInstrumentSearchTypesRequest
                            {
                                CredentialsHeader = credentials
                            };

                    // Call the API to get the response, handling API errors.
                    GetInstrumentSearchTypesResponse response;
                    try
                    {
                        response = proxy.GetInstrumentSearchTypes(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    foreach (var typeInfo in response.GetInstrumentSearchTypeResult)
                    {
                        writer.WriteLine("  {0} - Supported Identifier Types:",
                                     typeInfo.Type);
                        foreach (var identifier in typeInfo.SupportedIdentifierTypes)
                            writer.WriteLine("     {0} / {1}",
                                         identifier.Value, identifier.Description);
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}