﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Discoverability
{
    public class GetLegalEntityExtractionTypesSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Display sample description

                    writer.WriteLine("Invoking discoverability method...");

                    #endregion

                    #region Perform discoverability method call

                    // Create and initialize the request object. 
                    var request =
                        new GetLegalEntityExtractionTypesRequest
                            {
                                CredentialsHeader = credentials
                            };

                    // Call the API to get the response, handling API errors.
                    GetLegalEntityExtractionTypesResponse response;
                    try
                    {
                        response = proxy.GetLegalEntityExtractionTypes(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    foreach (var typeInfo in response.GetLegalEntityExtractionTypesResult)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     typeInfo.Value,
                                     typeInfo.Description);
                        writer.WriteLine("  - Maximum Legal Entities Allowed = {0}", typeInfo.MaximumLegalEntitiesAllowed);
                        writer.WriteLine("  - Time Period for Limits = {0}", typeInfo.TimePeriodForLimits);
                        writer.WriteLine("  - Maximum Legal Entities Allowed per Time Period = {0}",
                                     typeInfo.MaximumLegalEntitiesAllowedPerTimePeriod);
                        writer.WriteLine("  - Maximum Extractions Allowed per Time Period = {0}",
                                     typeInfo.MaximumExtractionsAllowedPerTimePeriod);
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}