﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Discoverability
{
    public class GetVersionSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    #region Display sample description

                    writer.WriteLine("Invoking discoverability method...");

                    #endregion

                    #region Perform discoverability method call

                    // Create and initialize the request object.
                    // NOTE: GetVersion does not require authentication, which is why the GetVersionRequest doesn't take a CredentialsHeader value
                    var request = new GetVersionRequest();

                    // Call the API to get the response, handling API errors.
                    GetVersionResponse response;
                    try
                    {
                        response = proxy.GetVersion(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("DSS Version = " + response.GetVersionResult.DssVersion);
                    writer.WriteLine("API Version = " + response.GetVersionResult.ApiVersion);

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}