using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Runtime.Serialization;

namespace DssApiSamples.Samples
{
    /// <summary>
    /// The <see cref="Exception"/> that is thrown by the <see cref="ExtractionUtility.EnsureOutputFields"/> method when 
    /// when an extraction contains one or more output fields that are no longer supported by the API.
    /// </summary>
    [Serializable]
    public class InvalidOutputFieldsException : Exception
    {
        #region Standard exception constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidOutputFieldsException"/> class.
        /// </summary>
        public InvalidOutputFieldsException() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidOutputFieldsException"/> class with a specified error message.
        /// </summary>
        public InvalidOutputFieldsException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidOutputFieldsException"/> class with a specified error message 
        /// and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        public InvalidOutputFieldsException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="InvalidOutputFieldsException"/> class with serialized data.
        /// </summary>
        public InvalidOutputFieldsException(SerializationInfo info, StreamingContext context) : base(info, context) { }

        #endregion
        
        /// <summary>
        /// Initalizes a new instance of a <see cref="InvalidOutputFieldsException"/> 
        /// with the specified list of invalid fields.
        /// </summary>
        public InvalidOutputFieldsException(string message, IEnumerable<string> fields)
            : base(message)
        {
            Fields = new ReadOnlyCollection<string>(fields.ToList());
        }

        /// <summary>
        /// Gets the list of invalid fields.
        /// </summary>
        public ReadOnlyCollection<string> Fields { get; private set; }
    }
}