using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.LegalEntity
{
    public class AllSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string identfierType = "ORG";
                    const string searchValue = "1228*";

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for all Legal Entities that:");
                    writer.WriteLine("- are {0} identifiers that begin with {1}", identfierType, searchValue);

                    #endregion

                    #region Search counterparties

                    // Create and initialize the request object. 
                    var request =
                        new SearchLegalEntitiesRequest
                            {
                                CredentialsHeader = credentials,
                                request = new LegalEntitySearchRequestAll
                                              {
                                                  // These are required values.
                                                  // NOTE: call GetLegalEntityIdentifierTypes operation to all Legal Entity identifier types.
                                                  IdentifierType = identfierType,
                                                  IdentifierSearchValue = searchValue
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    SearchLegalEntitiesResponse response;
                    try
                    {
                        response = proxy.SearchLegalEntities(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    // Display the response.
                    writer.WriteLine("{0} Legal Entities found:", response.SearchLegalEntitiesResult.LegalEntities.Length);
                    foreach (var legalEntity in response.SearchLegalEntitiesResult.LegalEntities)
                    {
                        writer.WriteLine("  {0}: {1}",
                                         legalEntity.IdentifierType,
                                         legalEntity.IdentifierValue);
                        writer.WriteLine("  - ORG ID = {0}", legalEntity.OrgId);
                        writer.WriteLine("  - Entity ID = {0}", legalEntity.EntityId);
                        writer.WriteLine("  - Description = {0}", legalEntity.Description);
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}