using System;
using System.Runtime.Serialization;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples
{
    /// <summary>
    /// The <see cref="Exception"/> that is thrown by the <see cref="ApiUtility.ConvertFaultException"/> method when 
    /// the SOAP fault represents a server error.
    /// </summary>
    [Serializable]
    public class ServerErrorException : Exception
    {
        #region Standard exception constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerErrorException"/> class.
        /// </summary>
        public ServerErrorException() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerErrorException"/> class with a specified error message.
        /// </summary>
        public ServerErrorException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerErrorException"/> class with a specified error message 
        /// and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        public ServerErrorException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerErrorException"/> class with serialized data.
        /// </summary>
        public ServerErrorException(SerializationInfo info, StreamingContext context) : base(info, context) { }

        #endregion

        /// <summary>
        /// Initalizes a new instance of a <see cref="ClientErrorException"/> 
        /// with the specified message and <see cref="ServerError"/>.
        /// </summary>
        public ServerErrorException(string message, ServerError error)
            : base(message)
        {
            Error = error;
        }

        /// <summary>
        /// Gets the <see cref="ServerError"/> that caused the exception.
        /// </summary>
        public ServerError Error { get; private set; }
    }
}